from unittest import TestCase
import uuid
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import Agent, AgentData, Prompt, LlmConfig, Model, Sampling, PromptExample, PromptOutput
from pydantic import ValidationError
from pygeai.core.common.exceptions import APIError


class TestAILabUpdateAgentIntegration(TestCase):    
    def setUp(self):
        """
        Set up the test environment.
        """
        self.ai_lab_manager = AILabManager(alias="beta")
        self.project_id = "be4889df-cacc-4e6f-b3bb-153c4ac0d168"

        load_agent = self.__load_agent()
        self.agent_to_update = load_agent["agent"]
        self.random_str = load_agent["random_str"]

    def __load_agent(self):
        random_str = str(uuid.uuid4())
        agent = Agent(
            id="b4b09935-2ad2-42c0-bd55-1ee6fa4b6034",
            name=f"UpdatedAgent{random_str}",
            access_scope="public",
            public_name=f"public_{random_str}",
            job_description=f"SummarizerAgent{random_str}",
            avatar_image="https://www.shareicon.net/data/128x128/2016/11/09/851442_logo_512x512.png",
            description=f"Agent that summarized documents. {random_str}",
            agent_data=AgentData(
                prompt=Prompt(
                    instructions="the user will provide a document, you must return a summary of the document.",
                    inputs=["text", "avoid slang indicator"],
                    outputs=[
                        PromptOutput(key="translated_text", description="translated text, with slang or not depending on the indication. in plain text.")
                    ],
                    examples=[
                        PromptExample(input_data="hola mundo [no-slang]", output='{"translated_text":"hello world"}'),
                        PromptExample(input_data="esto es una prueba pincheguey [keep-slang]", output='{"translated_text":"this is a test pal"}')
                    ]
                ),
                llm_config=LlmConfig(
                    max_tokens=1800,
                    timeout=0,
                    sampling=Sampling(temperature=0.3, top_k=0, top_p=0)
                ),
                models=[Model(name="openai/gpt-4-turbo-preview")]
            )
        )

        return { 
            "agent": agent,
            "random_str": random_str 
        }


    def __update_agent(self, agent=None, automatic_publish=False, upsert=False):
        """
        Helper to create an agent with the current project_id and ai_lab_manager.
        If automatic_publish is None, do not pass it (useful for tests that omit it).
        """
        return self.ai_lab_manager.update_agent(
            project_id=self.project_id,
            agent=self.agent_to_update if agent is None else agent,
            automatic_publish=automatic_publish, 
            upsert=upsert
        )
    

    def test_update_agent(self):
        self.agent_to_update.agent_data.models[0].name = "gemini/gemini-1.5-flash-8b-exp-0827"
        self.agent_to_update.access_scope = "private"
        updated_agent = self.__update_agent()

        self.assertTrue(isinstance(updated_agent, Agent), "Expected a created agent")
       
        self.assertEqual(updated_agent.name, f"UpdatedAgent{self.random_str}")
        self.assertEqual(updated_agent.access_scope, "private")
        self.assertEqual(updated_agent.public_name, f"public_{self.random_str}")
        self.assertEqual(updated_agent.job_description, f"SummarizerAgent{self.random_str}",)
        self.assertEqual(updated_agent.description, f"Agent that summarized documents. {self.random_str}")

        self.assertEqual(updated_agent.agent_data.models[0].name, "gemini/gemini-1.5-flash-8b-exp-0827")
        self.assertTrue(updated_agent.is_draft, "gemini/gemini-1.5-flash-8b-exp-0827")

        self.agent_to_update.agent_data.models[0].name = "openai/gpt-4-turbo-preview"
        updated_agent = self.__update_agent()

    
    def test_update_agent_no_name(self):
        test_params = [ True, False ]
        self.agent_to_update.name = ""
        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                with self.assertRaises(APIError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)
                
                self.assertIn(
                    "Error received while updating agent: errors=[Error(id=2007, description='Agent name cannot be empty.')]",
                    str(exception.exception),
                    f"Expected error about missing agent name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )


    def test_update_agent_duplicated_name(self):
        test_params = [ True, False ]
        self.agent_to_update.name = "AritmeticaExpert"

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                with self.assertRaises(APIError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)
                
                self.assertIn(
                    "Agent already exists [name=AritmeticaExpert].",
                    str(exception.exception),
                    f"Expected error about duplicated agent name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )

    
    def test_update_agent_invalidName_name(self):
        test_params = [ True, False ]
        self.agent_to_update.name = f"{self.agent_to_update.name}:/"

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):

                with self.assertRaises(APIError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)

                self.assertIn(
                    "Invalid character in name (: is not allowed).",
                    str(exception.exception),
                    f"Expected error about invalid character (:) in name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )
                self.assertIn(
                    "Invalid character in name (/ is not allowed).",
                    str(exception.exception),
                    f"Expected error about invalid character (/) in name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )


    def test_update_agent_no_public_name(self):        
        test_params = [ True, False ]
        self.agent_to_update.public_name = ""

        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                with self.assertRaises(APIError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)

                self.assertIn(
                    "Agent publicName is required for agents with accessScope=public.",
                    str(exception.exception),
                    f"Expected error about missing public name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )


    def test_update_agent_duplicated_public_name(self):               
        test_params = [ True, False ]
        self.agent_to_update.public_name = "com.testing.geai.googlesummarizer"

        for auto_publish in test_params:
            with self.subTest(input=auto_publish): 
                with self.assertRaises(APIError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)

                self.assertIn(
                    "Agent already exists [publicName=com.testing.geai.googlesummarizer].",
                    str(exception.exception),
                    f"Expected error about duplicated public name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )

    
    def test_update_agent_invalid_public_name(self):        
        test_params = [ True, False ]
        self.agent_to_update.public_name = self.agent_to_update.public_name.replace("_", "/")

        for auto_publish in test_params:
            with self.subTest(input=auto_publish): 
                with self.assertRaises(APIError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)

                self.assertIn(
                    "Invalid public name, it can only contain lowercase letters, numbers, periods (.), dashes (-), and underscores (_). Please remove any other characters.",
                    str(exception.exception),
                    f"Expected error about invalid public name when autopublish is {'enabled' if auto_publish else 'disabled'} was not returned."
                )


    def test_update_agent_no_prompt_instructions(self):
        test_params = [ True, False ]
        self.agent_to_update.agent_data.prompt.instructions = ""

        for auto_publish in test_params:
            with self.subTest(input=auto_publish): 
                with self.assertRaises(ValidationError) as exception:
                    self.__update_agent(automatic_publish=auto_publish)
                
                self.assertIn(
                    "instructions",
                    str(exception.exception),
                    f"Expected a validation error about allowed values for instructions when autopublish is {'enabled' if auto_publish else 'disabled'}"
                )
                self.assertIn(
                    "Input should be a valid string [type=string_type, input_value=None, input_type=NoneType]",
                    str(exception.exception),
                    f"Expected a validation error about allowed values for instructions when autopublish is {'enabled' if auto_publish else 'disabled'}"
                )

    
    def test_update_agent_no_model(self):        
        test_params = [ True, False ]
        self.agent_to_update.agent_data.models[0].name = ""
        
        for auto_publish in test_params:
            with self.subTest(input=auto_publish):
                if auto_publish == False:
                    with self.assertRaises(ValidationError) as exception:
                        self.__update_agent(automatic_publish=auto_publish)
                    error_msg = str(exception.exception)
                    
                    self.assertIn(
                        "name",
                        error_msg,
                        "Expected a validation error about empty model name" 
                    )
                    self.assertIn(
                        "Input should be a valid string [type=string_type, input_value=None, input_type=NoneType]",
                        error_msg,
                        "Expected a validation error about empty model name" 
                    )
                else:                 
                    with self.assertRaises(APIError) as exception:
                        self.__update_agent(automatic_publish=auto_publish)
                    error_msg = str(exception.exception)
                    self.assertIn(
                        "description='Model not found [name=]",
                        error_msg,
                        "Expected a validation error about allowed values for model name" 
                    )

    def test_update_agent_autopublish(self):        
        updated_agent = self.__update_agent(automatic_publish=True)

        self.assertEqual(
            updated_agent.name, 
            f"UpdatedAgent{self.random_str}",
            "Expected agent name to remain unchanged after update."
        )
        self.assertFalse(
            updated_agent.is_draft,
            "Expected agent to be published when autopublish is True, but it is still a draft."
        )


    def test_update_agent_autopublish_private_scope(self):
        self.agent_to_update.access_scope = "private"

        updated_agent = self.__update_agent(automatic_publish=True)
        self.assertFalse(updated_agent.is_draft, "Expected the agent to be published automatically even with private scope")
    


import unittest
from unittest.mock import MagicMock
from pygeai.core.handlers import ErrorHandler
from pygeai.core.base.mappers import ErrorMapper


class TestErrorHandler(unittest.TestCase):
    """
    python -m unittest pygeai.tests.core.test_handlers.TestErrorHandler
    """

    def setUp(self):
        # Mock the ErrorMapper methods to avoid actual mapping logic in tests
        self.error_mapper_map_to_error_list_response = MagicMock(return_value={"error_list": "mapped_errors"})
        self.error_mapper_map_to_error = MagicMock(return_value={"single_error": "mapped_error"})
        ErrorMapper.map_to_error_list_response = self.error_mapper_map_to_error_list_response
        ErrorMapper.map_to_error = self.error_mapper_map_to_error

    def test_has_errors_with_errors_key(self):
        response = {"errors": [{"code": "123", "message": "Error message"}]}

        result = ErrorHandler.has_errors(response)

        self.assertTrue(result)

    def test_has_errors_with_error_key(self):
        response = {"error": {"code": "456", "message": "Single error"}}

        result = ErrorHandler.has_errors(response)

        self.assertTrue(result)

    def test_has_errors_without_error_keys(self):
        response = {"data": "some data"}

        result = ErrorHandler.has_errors(response)

        self.assertFalse(result)

    def test_extract_error_with_errors_key(self):
        response = {"errors": [{"code": "123", "message": "Error message"}]}

        result = ErrorHandler.extract_error(response)

        self.error_mapper_map_to_error_list_response.assert_called_once_with(response)
        self.assertEqual(result, {"error_list": "mapped_errors"})
        self.error_mapper_map_to_error.assert_not_called()

    def test_extract_error_with_error_key(self):
        response = {"error": {"code": "456", "message": "Single error"}}

        result = ErrorHandler.extract_error(response)

        self.error_mapper_map_to_error.assert_called_once_with(response.get('error'))
        self.assertEqual(result, {"single_error": "mapped_error"})
        self.error_mapper_map_to_error_list_response.assert_not_called()

    def test_extract_error_without_error_keys(self):
        response = {"data": "some data"}

        result = ErrorHandler.extract_error(response)

        self.assertEqual(result, response)
        self.error_mapper_map_to_error_list_response.assert_not_called()
        self.error_mapper_map_to_error.assert_not_called()




import unittest
from unittest.mock import patch, Mock

from pygeai.cli.commands import Option
from pygeai.cli.commands.usage_limits import (
    show_help,
    set_organization_usage_limit,
    get_organization_latest_usage_limit,
    get_all_usage_limits_from_organization,
    delete_usage_limit_from_organization,
    update_organization_usage_limit,
    set_project_usage_limit,
    get_all_usage_limits_from_project,
    get_latest_usage_limit_from_project,
    get_active_usage_limit_from_project,
    delete_usage_limit_from_project,
    update_project_usage_limit
)
from pygeai.core.common.exceptions import MissingRequirementException


class TestUsageLimitsCommands(unittest.TestCase):
    """
    python -m unittest pygeai.tests.cli.commands.test_usage_limits.TestUsageLimitsCommands
    """
    def setUp(self):
        # Helper to create Option objects for testing
        self.mock_option = lambda name, value: (Option(name, [f"--{name}"], f"Description for {name}", True), value)

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.build_help_text')
    def test_show_help(self, mock_build_help, mock_write_stdout):
        mock_help_text = "Mocked help text"
        mock_build_help.return_value = mock_help_text

        show_help()

        mock_build_help.assert_called_once()
        mock_write_stdout.assert_called_once_with(mock_help_text)

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_set_organization_usage_limit_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.set_organization_usage_limit.return_value = {"status": "set"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("subscription_type", "basic"),
            self.mock_option("usage_unit", "requests"),
            self.mock_option("soft_limit", "100"),
            self.mock_option("hard_limit", "200"),
            self.mock_option("renewal_status", "monthly")
        ]

        set_organization_usage_limit(option_list)

        mock_instance.set_organization_usage_limit.assert_called_once_with(
            organization="org123",
            usage_limit={
                "subscriptionType": "basic",
                "usageUnit": "requests",
                "softLimit": "100",
                "hardLimit": "200",
                "renewalStatus": "monthly"
            }
        )
        mock_write_stdout.assert_called_once_with("Organization usage limit: \n{'status': 'set'}")

    def test_set_organization_usage_limit_missing_organization(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            set_organization_usage_limit(option_list)

        self.assertEqual(str(context.exception), "Cannot set usage limit for organization without organization ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_get_organization_latest_usage_limit_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_organization_latest_usage_limit.return_value = {"limit": "latest"}
        option_list = [self.mock_option("organization", "org123")]

        get_organization_latest_usage_limit(option_list)

        mock_instance.get_organization_latest_usage_limit.assert_called_once_with(organization="org123")
        mock_write_stdout.assert_called_once_with("Organization usage limit: \n{'limit': 'latest'}")

    def test_get_organization_latest_usage_limit_missing_organization(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            get_organization_latest_usage_limit(option_list)

        self.assertEqual(str(context.exception), "Cannot get latest usage limit for organization without organization ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_get_all_usage_limits_from_organization_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_all_usage_limits_from_organization.return_value = {"limits": ["limit1", "limit2"]}
        option_list = [self.mock_option("organization", "org123")]

        get_all_usage_limits_from_organization(option_list)

        mock_instance.get_all_usage_limits_from_organization.assert_called_once_with(organization="org123")
        mock_write_stdout.assert_called_once_with("Organization usage limits: \n{'limits': ['limit1', 'limit2']}")

    def test_get_all_usage_limits_from_organization_missing_organization(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            get_all_usage_limits_from_organization(option_list)

        self.assertEqual(str(context.exception), "Cannot get all usage limits for organization without organization ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_delete_usage_limit_from_organization_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.delete_usage_limit_from_organization.return_value = {"status": "deleted"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("limit_id", "limit456")
        ]

        delete_usage_limit_from_organization(option_list)

        mock_instance.delete_usage_limit_from_organization.assert_called_once_with(
            organization="org123",
            limit_id="limit456"
        )
        mock_write_stdout.assert_called_once_with("Deleted usage limit: \n{'status': 'deleted'}")

    def test_delete_usage_limit_from_organization_missing_fields(self):
        option_list = [self.mock_option("organization", "org123")]

        with self.assertRaises(MissingRequirementException) as context:
            delete_usage_limit_from_organization(option_list)

        self.assertEqual(str(context.exception), "Cannot delete usage limit for organization without organization ID and limit ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_update_organization_usage_limit_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.set_organization_hard_limit.return_value = {"status": "updated_hard"}
        mock_instance.set_organization_soft_limit.return_value = {"status": "updated_soft"}
        mock_instance.set_organization_renewal_status.return_value = {"status": "updated_renewal"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("limit_id", "limit456"),
            self.mock_option("hard_limit", "200"),
            self.mock_option("soft_limit", "100"),
            self.mock_option("renewal_status", "monthly")
        ]

        update_organization_usage_limit(option_list)

        mock_instance.set_organization_hard_limit.assert_called_once_with(
            organization="org123",
            limit_id="limit456",
            hard_limit="200"
        )
        mock_instance.set_organization_soft_limit.assert_called_once_with(
            organization="org123",
            limit_id="limit456",
            soft_limit="100"
        )
        mock_instance.set_organization_renewal_status.assert_called_once_with(
            organization="org123",
            limit_id="limit456",
            renewal_status="monthly"
        )

    def test_update_organization_usage_limit_missing_fields(self):
        option_list = [self.mock_option("organization", "org123")]

        with self.assertRaises(MissingRequirementException) as context:
            update_organization_usage_limit(option_list)

        self.assertEqual(str(context.exception), "Cannot update usage limit for organization without organization ID and limit ID")

    def test_update_organization_usage_limit_missing_update_values(self):
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("limit_id", "limit456")
        ]

        with self.assertRaises(MissingRequirementException) as context:
            update_organization_usage_limit(option_list)

        self.assertEqual(str(context.exception), "At least one of the following parameters must be define to update usage limit: --soft-limit, --hard-limit or --renewal-status")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_set_project_usage_limit_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.set_project_usage_limit.return_value = {"status": "set"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456"),
            self.mock_option("subscription_type", "premium"),
            self.mock_option("usage_unit", "tokens"),
            self.mock_option("soft_limit", "500"),
            self.mock_option("hard_limit", "1000"),
            self.mock_option("renewal_status", "yearly")
        ]

        set_project_usage_limit(option_list)

        mock_instance.set_project_usage_limit.assert_called_once_with(
            organization="org123",
            project="proj456",
            usage_limit={
                "subscriptionType": "premium",
                "usageUnit": "tokens",
                "softLimit": "500",
                "hardLimit": "1000",
                "renewalStatus": "yearly"
            }
        )
        mock_write_stdout.assert_called_once_with("Project usage limit: \n{'status': 'set'}")

    def test_set_project_usage_limit_missing_fields(self):
        option_list = [self.mock_option("organization", "org123")]

        with self.assertRaises(MissingRequirementException) as context:
            set_project_usage_limit(option_list)

        self.assertEqual(str(context.exception), "Cannot set usage limit for project without organization and project ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_get_all_usage_limits_from_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_all_usage_limits_from_project.return_value = {"limits": ["limit1", "limit2"]}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456")
        ]

        get_all_usage_limits_from_project(option_list)

        mock_instance.get_all_usage_limits_from_project.assert_called_once_with(
            organization="org123",
            project="proj456"
        )
        mock_write_stdout.assert_called_once_with("Project usage limits: \n{'limits': ['limit1', 'limit2']}")

    def test_get_all_usage_limits_from_project_missing_fields(self):
        option_list = [self.mock_option("organization", "org123")]

        with self.assertRaises(MissingRequirementException) as context:
            get_all_usage_limits_from_project(option_list)

        self.assertEqual(str(context.exception), "Cannot get usage limits for project without organization and project ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_get_latest_usage_limit_from_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_latest_usage_limit_from_project.return_value = {"limit": "latest"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456")
        ]

        get_latest_usage_limit_from_project(option_list)

        mock_instance.get_latest_usage_limit_from_project.assert_called_once_with(
            organization="org123",
            project="proj456"
        )
        mock_write_stdout.assert_called_once_with("Project's latest usage limit: \n{'limit': 'latest'}")

    def test_get_latest_usage_limit_from_project_missing_fields(self):
        option_list = [self.mock_option("organization", "org123")]

        with self.assertRaises(MissingRequirementException) as context:
            get_latest_usage_limit_from_project(option_list)

        self.assertEqual(str(context.exception), "Cannot get latest usage limit for project without organization and project ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_get_active_usage_limit_from_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_active_usage_limit_from_project.return_value = {"limit": "active"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456")
        ]

        get_active_usage_limit_from_project(option_list)

        mock_instance.get_active_usage_limit_from_project.assert_called_once_with(
            organization="org123",
            project="proj456"
        )
        mock_write_stdout.assert_called_once_with("Project's latest usage limit: \n{'limit': 'active'}")

    def test_get_active_usage_limit_from_project_missing_fields(self):
        option_list = [self.mock_option("organization", "org123")]

        with self.assertRaises(MissingRequirementException) as context:
            get_active_usage_limit_from_project(option_list)

        self.assertEqual(str(context.exception), "Cannot get active usage limit for project without organization and project ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_delete_usage_limit_from_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.delete_usage_limit_from_project.return_value = {"status": "deleted"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456"),
            self.mock_option("limit_id", "limit789")
        ]

        delete_usage_limit_from_project(option_list)

        mock_instance.delete_usage_limit_from_project.assert_called_once_with(
            organization="org123",
            project="proj456",
            limit_id="limit789"
        )
        mock_write_stdout.assert_called_once_with("Deleted usage limit: \n{'status': 'deleted'}")

    def test_delete_usage_limit_from_project_missing_fields(self):
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456")
        ]

        with self.assertRaises(MissingRequirementException) as context:
            delete_usage_limit_from_project(option_list)

        self.assertEqual(str(context.exception), "Cannot delete usage limit for project without organization, project and limit ID")

    @patch('pygeai.cli.commands.usage_limits.Console.write_stdout')
    @patch('pygeai.cli.commands.usage_limits.UsageLimitClient')
    def test_update_project_usage_limit_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.set_hard_limit_for_active_usage_limit_from_project.return_value = {"status": "updated_hard"}
        mock_instance.set_soft_limit_for_active_usage_limit_from_project.return_value = {"status": "updated_soft"}
        mock_instance.set_project_renewal_status.return_value = {"status": "updated_renewal"}
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456"),
            self.mock_option("limit_id", "limit789"),
            self.mock_option("hard_limit", "200"),
            self.mock_option("soft_limit", "100"),
            self.mock_option("renewal_status", "monthly")
        ]

        update_project_usage_limit(option_list)

        mock_instance.set_hard_limit_for_active_usage_limit_from_project.assert_called_once_with(
            organization="org123",
            project="proj456",
            limit_id="limit789",
            hard_limit="200"
        )
        mock_instance.set_soft_limit_for_active_usage_limit_from_project.assert_called_once_with(
            organization="org123",
            project="proj456",
            limit_id="limit789",
            soft_limit="100"
        )
        mock_instance.set_project_renewal_status.assert_called_once_with(
            organization="org123",
            project="proj456",
            limit_id="limit789",
            renewal_status="monthly"
        )

    def test_update_project_usage_limit_missing_fields(self):
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456")
        ]

        with self.assertRaises(MissingRequirementException) as context:
            update_project_usage_limit(option_list)

        self.assertEqual(str(context.exception), "Cannot update usage limit for project without organization ID, project ID and limit ID")

    def test_update_project_usage_limit_missing_update_values(self):
        option_list = [
            self.mock_option("organization", "org123"),
            self.mock_option("project", "proj456"),
            self.mock_option("limit_id", "limit789")
        ]

        with self.assertRaises(MissingRequirementException) as context:
            update_project_usage_limit(option_list)

        self.assertEqual(str(context.exception), "At least one of the following parameters must be define to update usage limit: --soft-limit, --hard-limit or --renewal-status")




import logging

from typing import Any

from starlette.applications import Starlette
from starlette.routing import Route

from a2a.server.apps.jsonrpc.jsonrpc_app import (
    CallContextBuilder,
    JSONRPCApplication,
)
from a2a.server.request_handlers.jsonrpc_handler import RequestHandler
from a2a.types import AgentCard


logger = logging.getLogger(__name__)


class A2AStarletteApplication(JSONRPCApplication):
    """A Starlette application implementing the A2A protocol server endpoints.

    Handles incoming JSON-RPC requests, routes them to the appropriate
    handler methods, and manages response generation including Server-Sent Events
    (SSE).
    """

    def __init__(
        self,
        agent_card: AgentCard,
        http_handler: RequestHandler,
        extended_agent_card: AgentCard | None = None,
        context_builder: CallContextBuilder | None = None,
    ):
        """Initializes the A2AStarletteApplication.

        Args:
            agent_card: The AgentCard describing the agent's capabilities.
            http_handler: The handler instance responsible for processing A2A
              requests via http.
            extended_agent_card: An optional, distinct AgentCard to be served
              at the authenticated extended card endpoint.
            context_builder: The CallContextBuilder used to construct the
              ServerCallContext passed to the http_handler. If None, no
              ServerCallContext is passed.
        """
        super().__init__(
            agent_card=agent_card,
            http_handler=http_handler,
            extended_agent_card=extended_agent_card,
            context_builder=context_builder,
        )

    def routes(
        self,
        agent_card_url: str = '/.well-known/agent.json',
        rpc_url: str = '/',
        extended_agent_card_url: str = '/agent/authenticatedExtendedCard',
    ) -> list[Route]:
        """Returns the Starlette Routes for handling A2A requests.

        Args:
            agent_card_url: The URL path for the agent card endpoint.
            rpc_url: The URL path for the A2A JSON-RPC endpoint (POST requests).
            extended_agent_card_url: The URL for the authenticated extended agent card endpoint.

        Returns:
            A list of Starlette Route objects.
        """
        app_routes = [
            Route(
                rpc_url,
                self._handle_requests,
                methods=['POST'],
                name='a2a_handler',
            ),
            Route(
                agent_card_url,
                self._handle_get_agent_card,
                methods=['GET'],
                name='agent_card',
            ),
        ]

        if self.agent_card.supportsAuthenticatedExtendedCard:
            app_routes.append(
                Route(
                    extended_agent_card_url,
                    self._handle_get_authenticated_extended_agent_card,
                    methods=['GET'],
                    name='authenticated_extended_agent_card',
                )
            )
        return app_routes

    def build(
        self,
        agent_card_url: str = '/.well-known/agent.json',
        rpc_url: str = '/',
        extended_agent_card_url: str = '/agent/authenticatedExtendedCard',
        **kwargs: Any,
    ) -> Starlette:
        """Builds and returns the Starlette application instance.

        Args:
            agent_card_url: The URL path for the agent card endpoint.
            rpc_url: The URL path for the A2A JSON-RPC endpoint (POST requests).
            extended_agent_card_url: The URL for the authenticated extended agent card endpoint.
            **kwargs: Additional keyword arguments to pass to the Starlette constructor.

        Returns:
            A configured Starlette application instance.
        """
        app_routes = self.routes(
            agent_card_url=agent_card_url,
            rpc_url=rpc_url,
            extended_agent_card_url=extended_agent_card_url,
        )
        if 'routes' in kwargs:
            kwargs['routes'].extend(app_routes)
        else:
            kwargs['routes'] = app_routes

        return Starlette(**kwargs)



from typing import List

from pygeai.lab.models import ReasoningStrategy, LocalizedDescription, ReasoningStrategyList


class ReasoningStrategyMapper:
    @classmethod
    def _map_localized_descriptions(cls, descriptions_data: List[dict]) -> List[LocalizedDescription]:
        """
        Maps a list of localized description dictionaries to a list of LocalizedDescription objects.

        :param descriptions_data: List[dict] - List of dictionaries containing language and description.
        :return: List[LocalizedDescription] - List of mapped LocalizedDescription objects.
        """
        return [
            LocalizedDescription(
                language=desc.get("language"),
                description=desc.get("description")
            )
            for desc in descriptions_data
        ]

    @classmethod
    def map_to_reasoning_strategy(cls, data: dict) -> ReasoningStrategy:
        """
        Maps a dictionary to a ReasoningStrategy object with explicit field mapping.

        :param data: dict - The dictionary containing reasoning strategy details.
        :return: ReasoningStrategy - The mapped ReasoningStrategy object.
        """
        strategy_data = data.get("strategyDefinition", data)
        name = strategy_data.get("name")
        system_prompt = strategy_data.get("systemPrompt")
        access_scope = strategy_data.get("accessScope")
        type_ = strategy_data.get("type")
        localized_descriptions_data = strategy_data.get("localizedDescriptions")
        localized_descriptions = cls._map_localized_descriptions(localized_descriptions_data) if localized_descriptions_data else None
        id = strategy_data.get("id")

        return ReasoningStrategy(
            name=name,
            system_prompt=system_prompt,
            access_scope=access_scope,
            type=type_,
            localized_descriptions=localized_descriptions,
            id=id
        )

    @classmethod
    def map_to_reasoning_strategy_list(cls, data: dict) -> ReasoningStrategyList:
        strategy_list = []
        strategies = data.get("strategies") if isinstance(data, dict) else data if isinstance(data, list) else []
        if strategies and any(strategies):
            for strategy_data in strategies:
                strategy = cls.map_to_reasoning_strategy(strategy_data)
                strategy_list.append(strategy)

        return ReasoningStrategyList(strategies=strategy_list)


import unittest
from unittest.mock import patch, MagicMock
from json import JSONDecodeError
from pygeai.lab.strategies.clients import ReasoningStrategyClient
from pygeai.core.common.exceptions import InvalidAPIResponseException


class TestReasoningStrategyClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.lab.strategies.test_clients.TestReasoningStrategyClient
    """
    def setUp(self):
        with patch('pygeai.core.base.clients.BaseClient.__init__', return_value=None):
            self.client = ReasoningStrategyClient()
        self.mock_response = MagicMock()
        self.client.api_service = MagicMock()
        self.project_id = "project-123"
        self.reasoning_strategy_id = "strat-123"
        self.reasoning_strategy_name = "TestStrategy"

    def test_list_reasoning_strategies_success(self):
        expected_response = {"strategies": [{"id": "strat-1", "name": "Strategy1"}]}
        self.mock_response.json.return_value = expected_response
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.list_reasoning_strategies(
            name="Strategy1",
            start="0",
            count="50",
            allow_external=True,
            access_scope="public"
        )

        self.assertEqual(result, expected_response)
        self.client.api_service.get.assert_called_once()
        call_args = self.client.api_service.get.call_args
        params = call_args[1]['params']
        self.assertEqual(params['name'], "Strategy1")
        self.assertEqual(params['start'], "0")
        self.assertEqual(params['count'], "50")
        self.assertTrue(params['allowExternal'])
        self.assertEqual(params['accessScope'], "public")

    def test_list_reasoning_strategies_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Raw response text"
        self.mock_response.status_code = 500
        self.client.api_service.get.return_value = self.mock_response

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.list_reasoning_strategies()

        self.client.api_service.get.assert_called_once()
        self.assertIn("Unable to list reasoning strategies", str(context.exception))

    def test_list_reasoning_strategies_invalid_access_scope(self):
        with self.assertRaises(ValueError) as context:
            self.client.list_reasoning_strategies(access_scope="invalid")

        self.assertEqual(str(context.exception), "Access scope must be either 'public' or 'private'.")

    def test_create_reasoning_strategy_success(self):
        name = "TestStrategy"
        system_prompt = "Test system prompt"
        access_scope = "public"
        strategy_type = "addendum"
        localized_descriptions = [{"language": "english", "description": "Test description"}]
        automatic_publish = True
        expected_response = {"id": "strat-123", "name": name}
        self.mock_response.json.return_value = expected_response
        self.client.api_service.post.return_value = self.mock_response

        result = self.client.create_reasoning_strategy(
            project_id=self.project_id,
            name=name,
            system_prompt=system_prompt,
            access_scope=access_scope,
            strategy_type=strategy_type,
            localized_descriptions=localized_descriptions,
            automatic_publish=automatic_publish
        )

        self.assertEqual(result, expected_response)
        self.client.api_service.post.assert_called_once()
        call_args = self.client.api_service.post.call_args
        data = call_args[1]['data']['strategyDefinition']
        self.assertEqual(data['name'], name)
        self.assertEqual(data['systemPrompt'], system_prompt)
        self.assertEqual(data['accessScope'], access_scope)
        self.assertEqual(data['type'], strategy_type)
        self.assertEqual(data['localizedDescriptions'], localized_descriptions)
        self.assertIn("automaticPublish=true", call_args[1]['endpoint'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_create_reasoning_strategy_json_decode_error(self):
        name = "TestStrategy"
        system_prompt = "Test system prompt"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Raw response text"
        self.mock_response.status_code = 500
        self.client.api_service.post.return_value = self.mock_response

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.create_reasoning_strategy(
                project_id=self.project_id,
                name=name,
                system_prompt=system_prompt
            )

        self.client.api_service.post.assert_called_once()
        self.assertIn("Unable to create reasoning strategy", str(context.exception))

    def test_update_reasoning_strategy_success(self):
        name = "UpdatedStrategy"
        system_prompt = "Updated prompt"
        access_scope = "private"
        strategy_type = "addendum"
        localized_descriptions = [{"language": "english", "description": "Updated description"}]
        automatic_publish = True
        upsert = False
        expected_response = {"id": self.reasoning_strategy_id, "name": name}
        self.mock_response.json.return_value = expected_response
        self.client.api_service.put.return_value = self.mock_response

        result = self.client.update_reasoning_strategy(
            project_id=self.project_id,
            reasoning_strategy_id=self.reasoning_strategy_id,
            name=name,
            system_prompt=system_prompt,
            access_scope=access_scope,
            strategy_type=strategy_type,
            localized_descriptions=localized_descriptions,
            automatic_publish=automatic_publish,
            upsert=upsert
        )

        self.assertEqual(result, expected_response)
        self.client.api_service.put.assert_called_once()
        call_args = self.client.api_service.put.call_args
        data = call_args[1]['data']['strategyDefinition']
        self.assertEqual(data['name'], name)
        self.assertEqual(data['systemPrompt'], system_prompt)
        self.assertEqual(data['accessScope'], access_scope)
        self.assertEqual(data['type'], strategy_type)
        self.assertEqual(data['localizedDescriptions'], localized_descriptions)
        self.assertIn("automaticPublish=true", call_args[1]['endpoint'])
        headers = call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_update_reasoning_strategy_json_decode_error(self):
        name = "UpdatedStrategy"
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Raw response text"
        self.mock_response.status_code = 500
        self.client.api_service.put.return_value = self.mock_response

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.update_reasoning_strategy(
                project_id=self.project_id,
                reasoning_strategy_id=self.reasoning_strategy_id,
                name=name
            )

        self.client.api_service.put.assert_called_once()
        self.assertIn("Unable to update reasoning strategy", str(context.exception))

    def test_update_reasoning_strategy_invalid_access_scope(self):
        with self.assertRaises(ValueError) as context:
            self.client.update_reasoning_strategy(
                project_id=self.project_id,
                reasoning_strategy_id=self.reasoning_strategy_id,
                access_scope="invalid"
            )

        self.assertEqual(str(context.exception), "Access scope must be either 'public' or 'private'.")

    def test_update_reasoning_strategy_invalid_type(self):
        with self.assertRaises(ValueError) as context:
            self.client.update_reasoning_strategy(
                project_id=self.project_id,
                reasoning_strategy_id=self.reasoning_strategy_id,
                strategy_type="invalid"
            )

        self.assertEqual(str(context.exception), "Type must be 'addendum'.")

    def test_get_reasoning_strategy_success_with_id(self):
        expected_response = {"id": self.reasoning_strategy_id, "name": "TestStrategy"}
        self.mock_response.json.return_value = expected_response
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_reasoning_strategy(
            project_id=self.project_id,
            reasoning_strategy_id=self.reasoning_strategy_id
        )

        self.assertEqual(result, expected_response)
        self.client.api_service.get.assert_called_once()
        headers = self.client.api_service.get.call_args[1]['headers']
        self.assertEqual(headers['ProjectId'], self.project_id)

    def test_get_reasoning_strategy_success_with_name(self):
        expected_response = {"name": self.reasoning_strategy_name}
        self.mock_response.json.return_value = expected_response
        self.client.api_service.get.return_value = self.mock_response

        result = self.client.get_reasoning_strategy(
            project_id=self.project_id,
            reasoning_strategy_name=self.reasoning_strategy_name
        )

        self.assertEqual(result, expected_response)
        self.client.api_service.get.assert_called_once()

    def test_get_reasoning_strategy_json_decode_error(self):
        self.mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        self.mock_response.text = "Raw response text"
        self.mock_response.status_code = 500
        self.client.api_service.get.return_value = self.mock_response

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.get_reasoning_strategy(
                project_id=self.project_id,
                reasoning_strategy_id=self.reasoning_strategy_id
            )

        self.client.api_service.get.assert_called_once()
        self.assertIn("Unable to retrieve reasoning strategy", str(context.exception))

    def test_get_reasoning_strategy_invalid_input(self):
        with self.assertRaises(ValueError) as context:
            self.client.get_reasoning_strategy(project_id=self.project_id)

        self.assertEqual(str(context.exception), "Either reasoning_strategy_id or reasoning_strategy_name must be provided.")




import json
from json import JSONDecodeError

from pygeai import logger
from pygeai.core.base.clients import BaseClient
from pygeai.core.common.exceptions import InvalidAPIResponseException
from pygeai.core.llm.endpoints import GET_PROVIDER_LIST_V2, GET_PROVIDER_DATA_V2, GET_PROVIDER_MODELS_V2, \
    GET_MODEL_DATA_V2


class LlmClient(BaseClient):

    def get_provider_list(self) -> dict:
        logger.debug("Obtaining provider list")
        response = self.api_service.get(endpoint=GET_PROVIDER_LIST_V2)
        try:
            result = response.json()
            return result
        except JSONDecodeError as e:
            logger.error(f"Unable to obtain provider list: JSON parsing error (status {response.status_code}): {e}. Response: {response.text}")
            raise InvalidAPIResponseException(f"Unable to obtain provider list: {response.text}")

    def get_provider_data(self, provider_name: str) -> dict:
        endpoint = GET_PROVIDER_DATA_V2.format(providerName=provider_name)

        logger.debug(f"Obtaining provider data for {provider_name}")

        response = self.api_service.get(endpoint=endpoint)
        try:
            result = response.json()
            return result
        except JSONDecodeError as e:
            logger.error(f"Unable to obtain provider data for {provider_name}: JSON parsing error (status {response.status_code}): {e}. Response: {response.text}")
            raise InvalidAPIResponseException(f"Unable to obtain provider data for {provider_name}: {response.text}")

    def get_provider_models(self, provider_name: str) -> dict:
        endpoint = GET_PROVIDER_MODELS_V2.format(providerName=provider_name)

        logger.debug(f"Obtaining provider models for {provider_name}")

        response = self.api_service.get(endpoint=endpoint)
        try:
            result = response.json()
            return result
        except JSONDecodeError as e:
            logger.error(f"Unable to obtain provider models for {provider_name}: JSON parsing error (status {response.status_code}): {e}. Response: {response.text}")
            raise InvalidAPIResponseException(f"Unable to obtain provider models for {provider_name}: {response.text}")

    def get_model_data(
            self,
            provider_name: str,
            model_name: str = None,
            model_id: str = None
    ) -> dict:
        endpoint = GET_MODEL_DATA_V2.format(
            providerName=provider_name,
            modelNameOrId=model_name or model_id
        )

        logger.debug(f"Obtaining model data for {provider_name}/{model_name or model_id}")

        response = self.api_service.get(endpoint=endpoint)
        try:
            result = response.json()
            return result
        except JSONDecodeError as e:
            logger.error(f"Unable to obtain model data for {provider_name}/{model_name or model_id}: JSON parsing error (status {response.status_code}): {e}. Response: {response.text}")
            raise InvalidAPIResponseException(f"Unable to obtain model data for {provider_name}/{model_name or model_id}: {response.text}")



import unittest
from unittest.mock import patch, Mock
from pygeai.cli.commands.organization import (
    show_help,
    list_assistants,
    get_project_list,
    get_project_detail,
    create_project,
    update_project,
    delete_project,
    get_project_tokens,
    export_request_data,
    Option
)
from pygeai.core.common.exceptions import MissingRequirementException


class TestOrganizationCommands(unittest.TestCase):
    """
    python -m unittest pygeai.tests.cli.commands.test_organization.TestOrganizationCommands
    """
    def setUp(self):
        # Helper to create Option objects for testing
        self.mock_option = lambda name, value: (Option(name, [f"--{name}"], f"Description for {name}", True), value)

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.build_help_text')
    def test_show_help(self, mock_build_help, mock_write_stdout):
        mock_help_text = "Mocked help text"
        mock_build_help.return_value = mock_help_text

        show_help()

        mock_build_help.assert_called_once()
        mock_write_stdout.assert_called_once_with(mock_help_text)

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.PluginClient')
    def test_list_assistants_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.list_assistants.return_value = {"assistants": ["assistant1", "assistant2"]}
        option_list = [
            self.mock_option("organization_id", "org123"),
            self.mock_option("project_id", "proj456")
        ]

        list_assistants(option_list)

        mock_instance.list_assistants.assert_called_once_with(organization_id="org123", project_id="proj456")
        mock_write_stdout.assert_called_once_with("Assistant list: \n{'assistants': ['assistant1', 'assistant2']}")

    def test_list_assistants_missing_organization_id(self):
        option_list = [self.mock_option("project_id", "proj456")]

        with self.assertRaises(MissingRequirementException) as context:
            list_assistants(option_list)

        self.assertEqual(str(context.exception), "Organization ID and Project ID are required.")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_get_project_list_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_project_list.return_value = {"projects": ["project1", "project2"]}
        option_list = [
            self.mock_option("detail", "full"),
            self.mock_option("name", "test_project")
        ]

        get_project_list(option_list)

        mock_instance.get_project_list.assert_called_once_with("full", "test_project")
        mock_write_stdout.assert_called_once_with("Project list: \n{'projects': ['project1', 'project2']}")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_get_project_detail_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_project_data.return_value = {"id": "proj123", "name": "Test Project"}
        option_list = [self.mock_option("project_id", "proj123")]

        get_project_detail(option_list)

        mock_instance.get_project_data.assert_called_once_with(project_id="proj123")
        mock_write_stdout.assert_called_once_with("Project detail: \n{'id': 'proj123', 'name': 'Test Project'}")

    def test_get_project_detail_missing_project_id(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            get_project_detail(option_list)

        self.assertEqual(str(context.exception), "Cannot retrieve project detail without project-id")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_create_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.create_project.return_value = {"id": "proj123", "name": "New Project"}
        option_list = [
            self.mock_option("name", "New Project"),
            self.mock_option("admin_email", "admin@example.com"),
            self.mock_option("description", "A test project"),
            self.mock_option("subscription_type", "basic")
        ]

        create_project(option_list)

        mock_instance.create_project.assert_called_once_with("New Project", "admin@example.com", "A test project")
        mock_write_stdout.assert_called_once_with("New project: \n{'id': 'proj123', 'name': 'New Project'}")

    def test_create_project_missing_name_and_email(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            create_project(option_list)

        self.assertEqual(str(context.exception), "Cannot create project without name and administrator's email")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_update_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.update_project.return_value = {"id": "proj123", "name": "Updated Project"}
        option_list = [
            self.mock_option("project_id", "proj123"),
            self.mock_option("name", "Updated Project"),
            self.mock_option("description", "Updated description")
        ]

        update_project(option_list)

        mock_instance.update_project.assert_called_once_with("proj123", "Updated Project", "Updated description")
        mock_write_stdout.assert_called_once_with("Updated project: \n{'id': 'proj123', 'name': 'Updated Project'}")

    def test_update_project_missing_project_id_and_name(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            update_project(option_list)

        self.assertEqual(str(context.exception), "Cannot update project without project-id and/or name")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_delete_project_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.delete_project.return_value = {"status": "deleted"}
        option_list = [self.mock_option("project_id", "proj123")]

        delete_project(option_list)

        mock_instance.delete_project.assert_called_once_with("proj123")
        mock_write_stdout.assert_called_once_with("Deleted project: \n{'status': 'deleted'}")

    def test_delete_project_missing_project_id(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            delete_project(option_list)

        self.assertEqual(str(context.exception), "Cannot delete project without project-id")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_get_project_tokens_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.get_project_tokens.return_value = {"tokens": ["token1", "token2"]}
        option_list = [self.mock_option("project_id", "proj123")]

        get_project_tokens(option_list)

        mock_instance.get_project_tokens.assert_called_once_with("proj123")
        mock_write_stdout.assert_called_once_with("Project tokens: \n{'tokens': ['token1', 'token2']}")

    def test_get_project_tokens_missing_project_id(self):
        option_list = []

        with self.assertRaises(MissingRequirementException) as context:
            get_project_tokens(option_list)

        self.assertEqual(str(context.exception), "Cannot retrieve project tokens without project-id")

    @patch('pygeai.cli.commands.organization.Console.write_stdout')
    @patch('pygeai.cli.commands.organization.OrganizationClient')
    def test_export_request_data_success(self, mock_client, mock_write_stdout):
        mock_instance = Mock()
        mock_client.return_value = mock_instance
        mock_instance.export_request_data.return_value = {"data": ["request1", "request2"]}
        option_list = [
            self.mock_option("assistant_name", "assistant1"),
            self.mock_option("status", "completed"),
            self.mock_option("skip", "10"),
            self.mock_option("count", "20")
        ]

        export_request_data(option_list)

        mock_instance.export_request_data.assert_called_once_with("assistant1", "completed", "10", "20")
        mock_write_stdout.assert_called_once_with("Request data: \n{'data': ['request1', 'request2']}")





from pygeai.lab.managers import AILabManager
from pygeai.lab.models import (
    Agent, AgentData, Prompt, PromptExample, PromptOutput, LlmConfig, Sampling, Model, ModelList,
    Tool, ToolParameter,
    Task,
    AgenticProcess, KnowledgeBase, AgenticActivity, ArtifactSignal, UserSignal, Event, SequenceFlow,
    Variable, ProcessInstance, VariableList
)

manager = AILabManager()
project_id = "2ca6883f-6778-40bb-bcc1-85451fb11107"

created_entities = {
    "agent_id": None,
    "tool_id": None,
    "task_id": None,
    "process_id": None,
    "instance_id": None
}


def rollback():
    print("\n=== Initiating Rollback ===")
    if created_entities["instance_id"]:
        print(f"Deleting instance {created_entities['instance_id']}...")
        result = manager.abort_instance(instance_id=created_entities["instance_id"])
        print(f"Rollback: {result}")
    if created_entities["process_id"]:
        print(f"Deleting process {created_entities['process_id']}...")
        result = manager.delete_process(process_id=created_entities["process_id"])
        print(f"Rollback: {result}")
    if created_entities["task_id"]:
        print(f"Deleting task {created_entities['task_id']}...")
        result = manager.delete_task(task_id=created_entities["task_id"])
        print(f"Rollback: {result}")
    if created_entities["tool_id"]:
        print(f"Deleting tool {created_entities['tool_id']}...")
        result = manager.delete_tool(tool_id=created_entities["tool_id"])
        print(f"Rollback: {result}")
    if created_entities["agent_id"]:
        print(f"Deleting agent {created_entities['agent_id']}...")
        result = manager.delete_agent(agent_id=created_entities["agent_id"])
        print(f"Rollback: {result}")
    print("Rollback complete.")


def main():
    # Agent Flow
    print("=== Agent Flow ===")
    print("Creating agent 'AgenticTester' as draft...")
    agent = Agent(
        name="AgenticTester",
        access_scope="public",
        public_name="agentic_tester",
        job_description="Analyzes data and provides insights",
        description="An agent designed for testing agentic workflows",
        agent_data=AgentData(
            prompt=Prompt(
                instructions="Analyze the provided text and summarize it.",
                inputs=["text"],
                outputs=[PromptOutput(key="summary", description="A concise summary of the text")],
                examples=[PromptExample(input_data="The quick brown fox jumps.", output='{"summary": "Fox jumps."}')]
            ),
            llm_config=LlmConfig(
                max_tokens=1000,
                timeout=30,
                sampling=Sampling(temperature=0.7, top_k=40, top_p=0.9)
            ),
            strategy_name="Dynamic Prompting",
            models=ModelList(models=[
                Model(name="gpt-4"),
                Model(name="gpt-3.5-turbo")
            ])
        )
    )
    create_agent_result = manager.create_agent(agent=agent, automatic_publish=False)
    if isinstance(create_agent_result, Agent):
        print(f"Success: Created Agent: {create_agent_result.name}, ID: {create_agent_result.id}")
        created_entities["agent_id"] = create_agent_result.id
    else:
        print("Error: Agent creation failed:", create_agent_result)
        rollback()
        exit()

    print("Updating agent description...")
    agent.description = "Updated agent for testing workflows with enhanced capabilities"
    update_agent_result = manager.update_agent(agent=agent, automatic_publish=False)
    if isinstance(update_agent_result, Agent):
        print(f"Success: Updated Agent: {update_agent_result.description}")
    else:
        print("Error: Agent update failed:", update_agent_result)
        rollback()
        exit()

    print("Publishing agent revision '1'...")
    publish_agent_result = manager.publish_agent_revision(agent_id=created_entities["agent_id"], revision="1")
    if isinstance(publish_agent_result, Agent):
        print(f"Success: Published Agent Revision: {publish_agent_result.name}")
    else:
        print("Error: Agent publish failed:", publish_agent_result)
        rollback()
        exit()

    print("Retrieving latest agent version...")
    latest_agent = manager.get_agent(agent_id=created_entities["agent_id"])
    if isinstance(latest_agent, Agent):
        print(f"Success: Latest Agent: {latest_agent.name}, Description: {latest_agent.description}")
    else:
        print("Error: Agent retrieval failed:", latest_agent.errors)
        rollback()
        exit()

    # Tool Flow
    print("\n=== Tool Flow ===")
    print("Creating tool 'TestyTool' as draft...")
    tool = Tool(
        name="TestyTool",
        description="A tool for processing text data",
        parameters=[
            ToolParameter(key="input_text", data_type="String", description="Text to process", is_required=True),
            ToolParameter(key="max_length", data_type="Integer", description="Maximum length of output", is_required=False, value="100")
        ]
    )
    create_tool_result = manager.create_tool(tool=tool, automatic_publish=False)
    if isinstance(create_tool_result, Tool):
        print(f"Success: Created Tool: {create_tool_result.name}, ID: {create_tool_result.id}")
        created_entities["tool_id"] = create_tool_result.id
    else:
        print("Error: Tool creation failed:", create_tool_result)
        rollback()
        exit()

    print("Updating tool description...")
    tool.description = "Updated tool for enhanced text processing"
    update_tool_result = manager.update_tool(tool=tool, automatic_publish=False)
    if isinstance(update_tool_result, Tool):
        print(f"Success: Updated Tool: {update_tool_result.description}")
    else:
        print("Error: Tool update failed:", update_tool_result)
        rollback()
        exit()

    print("Publishing tool revision '1'...")
    publish_tool_result = manager.publish_tool_revision(tool_id=created_entities["tool_id"], revision="1")
    if isinstance(publish_tool_result, Tool):
        print(f"Success: Published Tool Revision: {publish_tool_result.name}")
    else:
        print("Error: Tool publish failed:", publish_tool_result)
        rollback()
        exit()

    print("Retrieving latest tool version...")
    latest_tool = manager.get_tool(tool_id=created_entities["tool_id"])
    if isinstance(latest_tool, Tool):
        print(f"Success: Latest Tool: {latest_tool.name}, Description: {latest_tool.description}")
    else:
        print("Error: Tool retrieval failed:", latest_tool.errors)
        rollback()
        exit()

    # Task Flow
    print("\n=== Task Flow ===")
    print("Creating task 'TestyTask' as draft...")
    task = Task(
        name="TestyTask",
        description="Processes text data using an agent",
        title_template="Text Processing Task #{{id}}"
    )
    create_task_result = manager.create_task(task=task, automatic_publish=False)
    if isinstance(create_task_result, Task):
        print(f"Success: Created Task: {create_task_result.name}, ID: {create_task_result.id}")
        created_entities["task_id"] = create_task_result.id
    else:
        print("Error: Task creation failed:", create_task_result)
        rollback()
        exit()

    # Assign created task to task
    task = create_task_result

    print("Updating task description...")
    task.description = "Updated task for text processing with agent collaboration"
    update_task_result = manager.update_task(task=task, automatic_publish=False)
    if isinstance(update_task_result, Task):
        print(f"Success: Updated Task: {update_task_result.description}")
    else:
        print("Error: Task update failed:", update_task_result)
        rollback()
        exit()

    print("Publishing task revision '1'...")
    publish_task_result = manager.publish_task_revision(task_id=created_entities["task_id"], revision="1")
    if isinstance(publish_task_result, Task):
        print(f"Success: Published Task Revision: {publish_task_result.name}")
    else:
        print("Error: Task publish failed:", publish_task_result)
        rollback()
        exit()

    print("Retrieving latest task version...")
    latest_task = manager.get_task(task_id=created_entities["task_id"])
    if isinstance(latest_task, Task):
        print(f"Success: Latest Task: {latest_task.name}, Description: {latest_task.description}")
    else:
        print("Error: Task retrieval failed:", latest_task.errors)
        rollback()
        exit()

    # Process Flow
    print("\n=== Process Flow ===")
    print("Creating process 'TestyProcess' as draft...")
    process = AgenticProcess(
        key="testy_process",
        name="TestyProcess",
        description="A process to analyze text data",
        kb=KnowledgeBase(name="text-analysis-kb", artifact_type_name=["text-data"]),
        agentic_activities=[
            AgenticActivity(
                key="activity1",
                name="Analyze Text",
                task_name="TestyTask",
                agent_name="AgenticTester",
                agent_revision_id=1
            )
        ],
        artifact_signals=[
            ArtifactSignal(key="text_upload", name="Text Upload", handling_type="C", artifact_type_name=["text-data"])
        ],
        user_signals=[
            UserSignal(key="analysis_complete", name="Analysis Complete")
        ],
        start_event=Event(key="start", name="Start"),
        end_event=Event(key="end", name="End"),
        sequence_flows=[
            SequenceFlow(key="flow1", source_key="start", target_key="text_upload"),
            SequenceFlow(key="flow2", source_key="text_upload", target_key="activity1"),
            SequenceFlow(key="flow3", source_key="activity1", target_key="analysis_complete"),
            SequenceFlow(key="flow4", source_key="analysis_complete", target_key="end")
        ]
    )
    create_process_result = manager.create_process(process=process, automatic_publish=False)
    if isinstance(create_process_result, AgenticProcess):
        print(f"Success: Created Process: {create_process_result.name}, ID: {create_process_result.id}")
        created_entities["process_id"] = create_process_result.id
    else:
        print("Error: Process creation failed:", create_process_result)
        rollback()
        exit()

    print("Updating process description...")
    process.description = "Updated process for advanced text analysis"
    update_process_result = manager.update_process(process=process, automatic_publish=False)
    if isinstance(update_process_result, AgenticProcess):
        print(f"Success: Updated Process: {update_process_result.description}")
    else:
        print("Error: Process update failed:", update_process_result)
        rollback()
        exit()

    print("Publishing process revision '1'...")
    publish_process_result = manager.publish_process_revision(process_id=created_entities["process_id"], revision="1")
    if isinstance(publish_process_result, AgenticProcess):
        print(f"Success: Published Process Revision: {publish_process_result.name}")
    else:
        print("Error: Process publish failed:", publish_process_result)
        rollback()
        exit()

    print("Retrieving latest process version...")
    latest_process = manager.get_process(process_id=created_entities["process_id"])
    if isinstance(latest_process, AgenticProcess):
        print(f"Success: Latest Process: {latest_process.name}, Description: {latest_process.description}")
    else:
        print("Error: Process retrieval failed:", latest_process.errors)
        rollback()
        exit()

    # Process Instance Flow
    print("\n=== Process Instance Flow ===")
    print("Starting process instance for 'TestyProcess'...")
    # variables = VariableList(variables=[Variable(key="input_text", value="Sample text for analysis")])
    # variables = [Variable(key="input_text", value="Sample text for analysis")]
    variables = []
    instance_result = manager.start_instance(
        project_id=project_id,
        process_name="TestyProcess",
        subject="Text Analysis Request",
        variables=variables
    )
    if isinstance(instance_result, ProcessInstance):
        print(f"Success: Started Instance: Process: {instance_result.process.name}, ID: {instance_result.id}")
        created_entities["instance_id"] = instance_result.id
    else:
        print("Error: Instance start failed:", instance_result)
        rollback()
        exit()

    print("Retrieving instance information...")
    instance_info = manager.get_instance(instance_id=created_entities["instance_id"])
    if isinstance(instance_info, ProcessInstance):
        print(f"Success: Instance Info: Process: {instance_info.process.name}, Subject: {instance_info.subject}")
    else:
        print("Error: Instance retrieval failed:", instance_info.errors)
        rollback()
        exit()

    print("Retrieving instance history...")
    history = manager.get_instance_history(instance_id=created_entities["instance_id"])
    if isinstance(history, dict):
        print(f"Success: Instance History: {history}")
    else:
        print("Error: History retrieval failed:", history.errors)
        rollback()
        exit()

    print("\n=== Process Completed Successfully ===")


if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        breakpoint()
        rollback()
        print(f"\n# Critical error: {e}")


from pygeai.lab.managers import AILabManager
from pygeai.lab.models import FilterSettings, AgenticProcessList

manager = AILabManager()
filter_settings = FilterSettings(id="30a2ab8c-7503-4220-b6a0-77f3ee40d365", start="0", count="100", allow_drafts=True)
result = manager.list_processes(filter_settings=filter_settings)

print(f"Found {len(result.processes)} processes:")
for proc in result.processes:
    print(f"Name: {proc.name}, ID: {proc.id}")



import uuid

from a2a.server.context import ServerCallContext
from a2a.types import (
    InvalidParamsError,
    Message,
    MessageSendConfiguration,
    MessageSendParams,
    Task,
)
from a2a.utils import get_message_text
from a2a.utils.errors import ServerError


class RequestContext:
    """Request Context.

    Holds information about the current request being processed by the server,
    including the incoming message, task and context identifiers, and related
    tasks.
    """

    def __init__(  # noqa: PLR0913
        self,
        request: MessageSendParams | None = None,
        task_id: str | None = None,
        context_id: str | None = None,
        task: Task | None = None,
        related_tasks: list[Task] | None = None,
        call_context: ServerCallContext | None = None,
    ):
        """Initializes the RequestContext.

        Args:
            request: The incoming `MessageSendParams` request payload.
            task_id: The ID of the task explicitly provided in the request or path.
            context_id: The ID of the context explicitly provided in the request or path.
            task: The existing `Task` object retrieved from the store, if any.
            related_tasks: A list of other tasks related to the current request (e.g., for tool use).
            call_context: The server call context associated with this request.
        """
        if related_tasks is None:
            related_tasks = []
        self._params = request
        self._task_id = task_id
        self._context_id = context_id
        self._current_task = task
        self._related_tasks = related_tasks
        self._call_context = call_context
        # If the task id and context id were provided, make sure they
        # match the request. Otherwise, create them
        if self._params:
            if task_id:
                self._params.message.taskId = task_id
                if task and task.id != task_id:
                    raise ServerError(InvalidParamsError(message='bad task id'))
            else:
                self._check_or_generate_task_id()
            if context_id:
                self._params.message.contextId = context_id
                if task and task.contextId != context_id:
                    raise ServerError(
                        InvalidParamsError(message='bad context id')
                    )
            else:
                self._check_or_generate_context_id()

    def get_user_input(self, delimiter: str = '\n') -> str:
        """Extracts text content from the user's message parts.

        Args:
            delimiter: The string to use when joining multiple text parts.

        Returns:
            A single string containing all text content from the user message,
            joined by the specified delimiter. Returns an empty string if no
            user message is present or if it contains no text parts.
        """
        if not self._params:
            return ''

        return get_message_text(self._params.message, delimiter)

    def attach_related_task(self, task: Task) -> None:
        """Attaches a related task to the context.

        This is useful for scenarios like tool execution where a new task
        might be spawned.

        Args:
            task: The `Task` object to attach.
        """
        self._related_tasks.append(task)

    @property
    def message(self) -> Message | None:
        """The incoming `Message` object from the request, if available."""
        return self._params.message if self._params else None

    @property
    def related_tasks(self) -> list[Task]:
        """A list of tasks related to the current request."""
        return self._related_tasks

    @property
    def current_task(self) -> Task | None:
        """The current `Task` object being processed."""
        return self._current_task

    @current_task.setter
    def current_task(self, task: Task) -> None:
        """Sets the current task object."""
        self._current_task = task

    @property
    def task_id(self) -> str | None:
        """The ID of the task associated with this context."""
        return self._task_id

    @property
    def context_id(self) -> str | None:
        """The ID of the conversation context associated with this task."""
        return self._context_id

    @property
    def configuration(self) -> MessageSendConfiguration | None:
        """The `MessageSendConfiguration` from the request, if available."""
        if not self._params:
            return None
        return self._params.configuration

    @property
    def call_context(self) -> ServerCallContext | None:
        """The server call context associated with this request."""
        return self._call_context

    def _check_or_generate_task_id(self) -> None:
        """Ensures a task ID is present, generating one if necessary."""
        if not self._params:
            return

        if not self._task_id and not self._params.message.taskId:
            self._params.message.taskId = str(uuid.uuid4())
        if self._params.message.taskId:
            self._task_id = self._params.message.taskId

    def _check_or_generate_context_id(self) -> None:
        """Ensures a context ID is present, generating one if necessary."""
        if not self._params:
            return

        if not self._context_id and not self._params.message.contextId:
            self._params.message.contextId = str(uuid.uuid4())
        if self._params.message.contextId:
            self._context_id = self._params.message.contextId



SET_ORGANIZATION_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/limits"	# POST -> Defines a new usage limit for an organization
GET_ORGANIZATION_LATEST_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/limits/latest"	 # GET -> Retrieves the latest usage limit defined for the organization
GET_ALL_ORGANIZATION_USAGE_LIMITS_V2 = "v2/usageLimits/organizations/{organization}/limits"	# GET -> Retrieves all usage limits defined for the organization
DELETE_ORGANIZATION_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/limits/{id}"	# DELETE -> Deletes a usage limit for the organization
SET_ORGANIZATION_HARD_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/limits/{id}/hardLimit"	# PUT -> Modifies the hardLimit of an active usage limit for the organization
SET_ORGANIZATION_SOFT_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/limits/{id}/softLimit"	 # PUT -> Modifies the softLimit of an active usage limit for the organization
SET_ORGANIZATION_RENEWAL_STATUS_V2 = "v2/usageLimits/organizations/{organization}/limits/{id}/renewalStatus"	 # PUT -> Modifies the renewalStatus of an active usage limit for the organization
SET_PROJECT_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits"	 # POST -> Defines a new usage limit for a project
GET_ALL_PROJECT_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits"	 # GET -> Retrieves all usage limits for a project
GET_LATEST_PROJECT_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits/latest"	 # GET -> Retrieves the latest usage limit defined for a project
GET_PROJECT_ACTIVE_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits/active"	 # GET -> Retrieves the active usage limit for a project
DELETE_PROJECT_USAGE_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits/{id}"    # DELETE -> Deletes an active usage limit for a project
SET_PROJECT_HARD_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits/{id}/hardLimit"  # PUT -> Modifies the hardLimit for an active usage limit for a project
SET_PROJECT_SOFT_LIMIT_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits/{id}/softLimit"  # PUT -> Modifies the softLimit for an active usage limit for a project
SET_PROJECT_RENEWAL_STATUS_V2 = "v2/usageLimits/organizations/{organization}/projects/{project}/limits/{id}/renewalStatus"  # PUT -> Modifies the renewalStatus for an active usage limit for a project



import logging

from collections.abc import AsyncIterable

from a2a.server.context import ServerCallContext
from a2a.server.request_handlers.request_handler import RequestHandler
from a2a.server.request_handlers.response_helpers import prepare_response_object
from a2a.types import (
    AgentCard,
    CancelTaskRequest,
    CancelTaskResponse,
    CancelTaskSuccessResponse,
    GetTaskPushNotificationConfigRequest,
    GetTaskPushNotificationConfigResponse,
    GetTaskPushNotificationConfigSuccessResponse,
    GetTaskRequest,
    GetTaskResponse,
    GetTaskSuccessResponse,
    InternalError,
    JSONRPCErrorResponse,
    Message,
    SendMessageRequest,
    SendMessageResponse,
    SendMessageSuccessResponse,
    SendStreamingMessageRequest,
    SendStreamingMessageResponse,
    SendStreamingMessageSuccessResponse,
    SetTaskPushNotificationConfigRequest,
    SetTaskPushNotificationConfigResponse,
    SetTaskPushNotificationConfigSuccessResponse,
    Task,
    TaskArtifactUpdateEvent,
    TaskNotFoundError,
    TaskPushNotificationConfig,
    TaskResubscriptionRequest,
    TaskStatusUpdateEvent,
)
from a2a.utils.errors import ServerError
from a2a.utils.helpers import validate
from a2a.utils.telemetry import SpanKind, trace_class


logger = logging.getLogger(__name__)


@trace_class(kind=SpanKind.SERVER)
class JSONRPCHandler:
    """Maps incoming JSON-RPC requests to the appropriate request handler method and formats responses."""

    def __init__(
        self,
        agent_card: AgentCard,
        request_handler: RequestHandler,
    ):
        """Initializes the JSONRPCHandler.

        Args:
            agent_card: The AgentCard describing the agent's capabilities.
            request_handler: The underlying `RequestHandler` instance to delegate requests to.
        """
        self.agent_card = agent_card
        self.request_handler = request_handler

    async def on_message_send(
        self,
        request: SendMessageRequest,
        context: ServerCallContext | None = None,
    ) -> SendMessageResponse:
        """Handles the 'message/send' JSON-RPC method.

        Args:
            request: The incoming `SendMessageRequest` object.
            context: Context provided by the server.

        Returns:
            A `SendMessageResponse` object containing the result (Task or Message)
            or a JSON-RPC error response if a `ServerError` is raised by the handler.
        """
        # TODO: Wrap in error handler to return error states
        try:
            task_or_message = await self.request_handler.on_message_send(
                request.params, context
            )
            return prepare_response_object(
                request.id,
                task_or_message,
                (Task, Message),
                SendMessageSuccessResponse,
                SendMessageResponse,
            )
        except ServerError as e:
            return SendMessageResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )

    @validate(
        lambda self: self.agent_card.capabilities.streaming,
        'Streaming is not supported by the agent',
    )
    async def on_message_send_stream(
        self,
        request: SendStreamingMessageRequest,
        context: ServerCallContext | None = None,
    ) -> AsyncIterable[SendStreamingMessageResponse]:
        """Handles the 'message/stream' JSON-RPC method.

        Yields response objects as they are produced by the underlying handler's stream.

        Args:
            request: The incoming `SendStreamingMessageRequest` object.
            context: Context provided by the server.

        Yields:
            `SendStreamingMessageResponse` objects containing streaming events
            (Task, Message, TaskStatusUpdateEvent, TaskArtifactUpdateEvent)
            or JSON-RPC error responses if a `ServerError` is raised.
        """
        try:
            async for event in self.request_handler.on_message_send_stream(
                request.params, context
            ):
                yield prepare_response_object(
                    request.id,
                    event,
                    (
                        Task,
                        Message,
                        TaskArtifactUpdateEvent,
                        TaskStatusUpdateEvent,
                    ),
                    SendStreamingMessageSuccessResponse,
                    SendStreamingMessageResponse,
                )
        except ServerError as e:
            yield SendStreamingMessageResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )

    async def on_cancel_task(
        self,
        request: CancelTaskRequest,
        context: ServerCallContext | None = None,
    ) -> CancelTaskResponse:
        """Handles the 'tasks/cancel' JSON-RPC method.

        Args:
            request: The incoming `CancelTaskRequest` object.
            context: Context provided by the server.

        Returns:
            A `CancelTaskResponse` object containing the updated Task or a JSON-RPC error.
        """
        try:
            task = await self.request_handler.on_cancel_task(
                request.params, context
            )
            if task:
                return prepare_response_object(
                    request.id,
                    task,
                    (Task,),
                    CancelTaskSuccessResponse,
                    CancelTaskResponse,
                )
            raise ServerError(error=TaskNotFoundError())
        except ServerError as e:
            return CancelTaskResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )

    async def on_resubscribe_to_task(
        self,
        request: TaskResubscriptionRequest,
        context: ServerCallContext | None = None,
    ) -> AsyncIterable[SendStreamingMessageResponse]:
        """Handles the 'tasks/resubscribe' JSON-RPC method.

        Yields response objects as they are produced by the underlying handler's stream.

        Args:
            request: The incoming `TaskResubscriptionRequest` object.
            context: Context provided by the server.

        Yields:
            `SendStreamingMessageResponse` objects containing streaming events
            or JSON-RPC error responses if a `ServerError` is raised.
        """
        try:
            async for event in self.request_handler.on_resubscribe_to_task(
                request.params, context
            ):
                yield prepare_response_object(
                    request.id,
                    event,
                    (
                        Task,
                        Message,
                        TaskArtifactUpdateEvent,
                        TaskStatusUpdateEvent,
                    ),
                    SendStreamingMessageSuccessResponse,
                    SendStreamingMessageResponse,
                )
        except ServerError as e:
            yield SendStreamingMessageResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )

    async def get_push_notification(
        self,
        request: GetTaskPushNotificationConfigRequest,
        context: ServerCallContext | None = None,
    ) -> GetTaskPushNotificationConfigResponse:
        """Handles the 'tasks/pushNotificationConfig/get' JSON-RPC method.

        Args:
            request: The incoming `GetTaskPushNotificationConfigRequest` object.
            context: Context provided by the server.

        Returns:
            A `GetTaskPushNotificationConfigResponse` object containing the config or a JSON-RPC error.
        """
        try:
            config = (
                await self.request_handler.on_get_task_push_notification_config(
                    request.params, context
                )
            )
            return prepare_response_object(
                request.id,
                config,
                (TaskPushNotificationConfig,),
                GetTaskPushNotificationConfigSuccessResponse,
                GetTaskPushNotificationConfigResponse,
            )
        except ServerError as e:
            return GetTaskPushNotificationConfigResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )

    @validate(
        lambda self: self.agent_card.capabilities.pushNotifications,
        'Push notifications are not supported by the agent',
    )
    async def set_push_notification(
        self,
        request: SetTaskPushNotificationConfigRequest,
        context: ServerCallContext | None = None,
    ) -> SetTaskPushNotificationConfigResponse:
        """Handles the 'tasks/pushNotificationConfig/set' JSON-RPC method.

        Requires the agent to support push notifications.

        Args:
            request: The incoming `SetTaskPushNotificationConfigRequest` object.
            context: Context provided by the server.

        Returns:
            A `SetTaskPushNotificationConfigResponse` object containing the config or a JSON-RPC error.

        Raises:
            ServerError: If push notifications are not supported by the agent
                (due to the `@validate` decorator).
        """
        try:
            config = (
                await self.request_handler.on_set_task_push_notification_config(
                    request.params, context
                )
            )
            return prepare_response_object(
                request.id,
                config,
                (TaskPushNotificationConfig,),
                SetTaskPushNotificationConfigSuccessResponse,
                SetTaskPushNotificationConfigResponse,
            )
        except ServerError as e:
            return SetTaskPushNotificationConfigResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )

    async def on_get_task(
        self,
        request: GetTaskRequest,
        context: ServerCallContext | None = None,
    ) -> GetTaskResponse:
        """Handles the 'tasks/get' JSON-RPC method.

        Args:
            request: The incoming `GetTaskRequest` object.
            context: Context provided by the server.

        Returns:
            A `GetTaskResponse` object containing the Task or a JSON-RPC error.
        """
        try:
            task = await self.request_handler.on_get_task(
                request.params, context
            )
            if task:
                return prepare_response_object(
                    request.id,
                    task,
                    (Task,),
                    GetTaskSuccessResponse,
                    GetTaskResponse,
                )
            raise ServerError(error=TaskNotFoundError())
        except ServerError as e:
            return GetTaskResponse(
                root=JSONRPCErrorResponse(
                    id=request.id, error=e.error if e.error else InternalError()
                )
            )



from typing import Optional
import requests as req
import json


class ResponseMock:
    def __init__(self, status_code: int, content: str, url: Optional[str], reason: str):
        self.status_code = status_code
        self.content = content
        self.url = url
        self.reason = reason

    def raise_for_status(self):
        if self.status_code >= 400:
            raise req.exceptions.HTTPError(f"{self.status_code} {self.reason}")

    def json(self):
        return json.loads(self.content)


from pygeai.core.models import Project, Organization
from pygeai.organization.limits.managers import UsageLimitManager


organization = Organization(id="4aa15b61-d3c7-4a5c-99b8-052d18a04ff2")
project = Project(id="1956c032-3c66-4435-acb8-6a06e52f819f")

limit_id = "6d95069b-bab5-4281-b432-a7f5b4290227"

manager = UsageLimitManager(alias="sdkorg", organization_id=organization.id)

response = manager.delete_usage_limit_from_organization(
    limit_id=limit_id
)

print(f"response: {response}")


from pygeai.lab.managers import AILabManager
from pygeai.lab.models import FilterSettings, Agent


manager = AILabManager()


result = manager.get_agent(
    agent_id="9716a0a1-5eab-4cc9-a611-fa2c3237c511"
)

print(f"Retrieved agent: {result.to_dict()}")

filter_settings = FilterSettings(
    revision="0",
    version="0",
    allow_drafts=False
)
result = manager.get_agent(
    agent_id="9716a0a1-5eab-4cc9-a611-fa2c3237c511",
    filter_settings=filter_settings
)

print(f"Retrieved agent: {result.to_dict()}")



import unittest
from json import JSONDecodeError
from unittest.mock import patch

from pygeai.organization.clients import OrganizationClient
from pygeai.core.common.exceptions import InvalidAPIResponseException
from pygeai.organization.endpoints import GET_ASSISTANT_LIST_V1, GET_PROJECT_LIST_V1, GET_PROJECT_V1, CREATE_PROJECT_V1, \
    UPDATE_PROJECT_V1, DELETE_PROJECT_V1, GET_PROJECT_TOKENS_V1, GET_REQUEST_DATA_V1


class TestOrganizationClient(unittest.TestCase):
    """
    python -m unittest pygeai.tests.organization.test_clients.TestOrganizationClient
    """

    def setUp(self):
        self.client = OrganizationClient()

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_assistant_list_success(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"assistants": [{"name": "assistant1"}, {"name": "assistant2"}]}

        result = self.client.get_assistant_list(detail="summary")

        mock_get.assert_called_once_with(endpoint=GET_ASSISTANT_LIST_V1, params={"detail": "summary"})
        self.assertIsNotNone(result)
        self.assertEqual(len(result['assistants']), 2)
        self.assertEqual(result['assistants'][0]['name'], "assistant1")
        self.assertEqual(result['assistants'][1]['name'], "assistant2")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_assistant_list_json_decode_error(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.get_assistant_list(detail="full")

        mock_get.assert_called_once_with(endpoint=GET_ASSISTANT_LIST_V1, params={"detail": "full"})
        self.assertEqual(str(context.exception), "Unable to get assistant list: Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_list_success(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"projects": [{"name": "project1"}, {"name": "project2"}]}

        result = self.client.get_project_list(detail="summary")

        mock_get.assert_called_once_with(endpoint=GET_PROJECT_LIST_V1, params={"detail": "summary"})
        self.assertIsNotNone(result)
        self.assertEqual(len(result['projects']), 2)
        self.assertEqual(result['projects'][0]['name'], "project1")
        self.assertEqual(result['projects'][1]['name'], "project2")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_list_with_name(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"projects": [{"name": "specific_project"}]}

        result = self.client.get_project_list(detail="full", name="specific_project")

        mock_get.assert_called_once_with(
            endpoint=GET_PROJECT_LIST_V1,
            params={"detail": "full", "name": "specific_project"}
        )
        self.assertIsNotNone(result)
        self.assertEqual(len(result['projects']), 1)
        self.assertEqual(result['projects'][0]['name'], "specific_project")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_list_json_decode_error(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.get_project_list(detail="full")

        mock_get.assert_called_once_with(endpoint=GET_PROJECT_LIST_V1, params={"detail": "full"})
        self.assertEqual(str(context.exception), "Unable to get project list: Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_data_success(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"project": {"id": "123", "name": "project1"}}

        result = self.client.get_project_data(project_id="123")

        mock_get.assert_called_once_with(endpoint=GET_PROJECT_V1.format(id="123"))
        self.assertIsNotNone(result)
        self.assertEqual(result['project']['name'], "project1")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_data_json_decode_error(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.get_project_data(project_id="123")

        mock_get.assert_called_once_with(endpoint=GET_PROJECT_V1.format(id="123"))
        self.assertEqual(str(context.exception), "Unable to get project data for ID '123': Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.post")
    def test_create_project_success(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.json.return_value = {"project": {"id": "123", "name": "project1"}}

        result = self.client.create_project(name="project1", email="admin@example.com", description="A test project")

        mock_post.assert_called_once_with(
            endpoint=CREATE_PROJECT_V1,
            data={
                "name": "project1",
                "administratorUserEmail": "admin@example.com",
                "description": "A test project"
            }
        )
        self.assertIsNotNone(result)
        self.assertEqual(result['project']['name'], "project1")

    @patch("pygeai.core.services.rest.ApiService.post")
    def test_create_project_with_usage_limit(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.json.return_value = {"project": {"id": "123", "name": "project1"}}

        usage_limit = {"type": "Requests", "threshold": 1000}
        result = self.client.create_project(
            name="project1", email="admin@example.com", description="A test project", usage_limit=usage_limit
        )

        mock_post.assert_called_once_with(
            endpoint=CREATE_PROJECT_V1,
            data={
                "name": "project1",
                "administratorUserEmail": "admin@example.com",
                "description": "A test project",
                "usageLimit": usage_limit
            }
        )
        self.assertIsNotNone(result)
        self.assertEqual(result['project']['name'], "project1")

    @patch("pygeai.core.services.rest.ApiService.post")
    def test_create_project_json_decode_error(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.create_project(name="project1", email="admin@example.com")

        mock_post.assert_called_once_with(
            endpoint=CREATE_PROJECT_V1,
            data={
                "name": "project1",
                "administratorUserEmail": "admin@example.com",
                "description": None
            }
        )
        self.assertEqual(str(context.exception), "Unable to create project with name 'project1': Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.put")
    def test_update_project_success(self, mock_put):
        mock_response = mock_put.return_value
        mock_response.json.return_value = {"project": {"id": "123", "name": "updated_project"}}

        result = self.client.update_project(project_id="123", name="updated_project", description="Updated description")

        mock_put.assert_called_once_with(
            endpoint=UPDATE_PROJECT_V1.format(id="123"),
            data={"name": "updated_project", "description": "Updated description"}
        )
        self.assertIsNotNone(result)
        self.assertEqual(result['project']['name'], "updated_project")

    @patch("pygeai.core.services.rest.ApiService.put")
    def test_update_project_json_decode_error(self, mock_put):
        mock_response = mock_put.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.update_project(project_id="123", name="updated_project")

        mock_put.assert_called_once_with(
            endpoint=UPDATE_PROJECT_V1.format(id="123"),
            data={"name": "updated_project", "description": None}
        )
        self.assertEqual(str(context.exception), "Unable to update project with ID '123': Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.delete")
    def test_delete_project_success(self, mock_delete):
        mock_response = mock_delete.return_value
        mock_response.json.return_value = {"status": "deleted"}

        result = self.client.delete_project(project_id="123")

        mock_delete.assert_called_once_with(endpoint=DELETE_PROJECT_V1.format(id="123"))
        self.assertIsNotNone(result)
        self.assertEqual(result['status'], "deleted")

    @patch("pygeai.core.services.rest.ApiService.delete")
    def test_delete_project_json_decode_error(self, mock_delete):
        mock_response = mock_delete.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.delete_project(project_id="123")

        mock_delete.assert_called_once_with(endpoint=DELETE_PROJECT_V1.format(id="123"))
        self.assertEqual(str(context.exception), "Unable to delete project with ID '123': Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_tokens_success(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"tokens": ["token1", "token2"]}

        result = self.client.get_project_tokens(project_id="123")

        mock_get.assert_called_once_with(endpoint=GET_PROJECT_TOKENS_V1.format(id="123"))
        self.assertIsNotNone(result)
        self.assertEqual(len(result['tokens']), 2)
        self.assertEqual(result['tokens'][0], "token1")
        self.assertEqual(result['tokens'][1], "token2")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_get_project_tokens_json_decode_error(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.get_project_tokens(project_id="123")

        mock_get.assert_called_once_with(endpoint=GET_PROJECT_TOKENS_V1.format(id="123"))
        self.assertEqual(str(context.exception), "Unable to get tokens for project with ID '123': Invalid JSON response")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_export_request_data_success(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"requests": [{"id": "1", "status": "pending"}]}

        result = self.client.export_request_data()

        mock_get.assert_called_once_with(
            endpoint=GET_REQUEST_DATA_V1,
            params={"assistantName": None, "status": None, "skip": 0, "count": 0}
        )
        self.assertIsNotNone(result)
        self.assertEqual(len(result['requests']), 1)
        self.assertEqual(result['requests'][0]['status'], "pending")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_export_request_data_with_params(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.json.return_value = {"requests": [{"id": "1", "status": "completed"}]}

        result = self.client.export_request_data(assistant_name="assistant1", status="completed", skip=10, count=5)

        mock_get.assert_called_once_with(
            endpoint=GET_REQUEST_DATA_V1,
            params={"assistantName": "assistant1", "status": "completed", "skip": 10, "count": 5}
        )
        self.assertIsNotNone(result)
        self.assertEqual(len(result['requests']), 1)
        self.assertEqual(result['requests'][0]['status'], "completed")

    @patch("pygeai.core.services.rest.ApiService.get")
    def test_export_request_data_json_decode_error(self, mock_get):
        mock_response = mock_get.return_value
        mock_response.status_code = 200
        mock_response.json.side_effect = JSONDecodeError("Invalid JSON", "", 0)
        mock_response.text = "Invalid JSON response"

        with self.assertRaises(InvalidAPIResponseException) as context:
            self.client.export_request_data(assistant_name="assistant1")

        mock_get.assert_called_once_with(
            endpoint=GET_REQUEST_DATA_V1,
            params={"assistantName": "assistant1", "status": None, "skip": 0, "count": 0}
        )
        self.assertEqual(str(context.exception), "Unable to export request data: Invalid JSON response")




from uuid import uuid4
from pygeai.chat.managers import ChatManager
from pygeai.core.files.responses import UploadFileResponse
from pygeai.core.files.managers import FileManager
from pygeai.core.files.models import UploadFile
from pygeai.core.models import ChatMessageList, ChatMessage, LlmSettings
from pygeai.assistant.managers import AssistantManager
from pygeai.core.services.llm.model import Model
from pygeai.core.services.llm.providers import Provider
from pygeai.core.models import WelcomeData, WelcomeDataFeature, WelcomeDataExamplePrompt, GuardrailSettings, ChatAssistant

project_id = "2ca6883f-6778-40bb-bcc1-85451fb11107"

file_manager = FileManager(
    organization_id="4aa15b61-d3c7-4a5c-99b8-052d18a04ff2",
    project_id=project_id
)
chat_manager = ChatManager()
assistant_manager = AssistantManager()

created_entities = {
    "assistant_id": None,
    "file_id": None
}


def rollback():
    print("\n=== Initiating Rollback ===")
    if created_entities["assistant_id"]:
        print(f"Deleting assistant {created_entities['assistant_id']}...")
        result = assistant_manager.delete_assistant(assistant_id=created_entities["assistant_id"])
        print(f"Rollback: {result}")
    if created_entities["file_id"]:
        print(f"Deleting file {created_entities['file_id']}...")
        result = file_manager.delete_file(file_id=created_entities["file_id"])
        print(f"Rollback: {result}")
    print("Rollback complete.")


def main():
    # File Upload Flow
    print("\n=== File Upload Flow ===")
    print("Uploading file 'test.txt'...")
    file_to_upload = UploadFile(
        name="test.txt",
        path="./test.txt",
        folder="summaries"
    )
    upload_result = file_manager.upload_file(file=file_to_upload)
    if isinstance(upload_result, UploadFileResponse) and upload_result.success:
        print(f"Success: Uploaded File: test.txt, ID: {upload_result.id}")
        created_entities["file_id"] = upload_result.id
    else:
        print("Error: File upload failed:", upload_result)
        rollback()
        exit()

    # Retrieve File Metadata
    print("Retrieving file metadata...")
    file_data = file_manager.get_file_data(file_id=created_entities["file_id"])
    if hasattr(file_data, "name"):
        file_name = file_data.name
        print(f"Success: Retrieved File Name: {file_name}")
    else:
        print("Error: File retrieval failed:", file_data)
        rollback()
        exit()

    # Assistant Flow
    print("\n=== Assistant Flow ===")
    print("Creating assistant 'FileSummaryAssistant'...")
    assistant_id = str(uuid4())
    guardrail_settings = GuardrailSettings(
        llm_output=True,
        input_moderation=True,
        prompt_injection=True
    )
    llm_settings = LlmSettings(
        provider_name=Provider.OPENAI,
        model_name=Model.OpenAI.GPT_4,
        temperature=0.7,
        max_tokens=1000,
        upload_files=True,
        guardrail_settings=guardrail_settings
    )
    welcome_data = WelcomeData(
        title="File Summary Assistant",
        description="An assistant for summarizing files",
        features=[
            WelcomeDataFeature(title="File Summarization", description="Summarizes content from uploaded text files"),
            WelcomeDataFeature(title="Quick Processing", description="Generates concise summaries in under 50 words")
        ],
        examples_prompt=[
            WelcomeDataExamplePrompt(
                title="Summarize File",
                description="Summarize the content of an uploaded file",
                prompt_text=f"Summarize the content of the file {{file:{file_name}}} in 50 words or less."
            )
        ]
    )
    assistant = ChatAssistant(
        id=assistant_id,
        name="FileSummaryAssistant2",
        description="An assistant for summarizing text file content",
        prompt="Read the provided text file by its ID and generate a concise summary in 50 words or less.",
        llm_settings=llm_settings,
        welcome_data=welcome_data
    )
    create_assistant_result = assistant_manager.create_assistant(assistant)
    create_assistant_result = create_assistant_result.assistant if hasattr(create_assistant_result, "assistant") else create_assistant_result
    if isinstance(create_assistant_result, ChatAssistant):
        print(f"Success: Created Assistant: {create_assistant_result.name}, ID: {create_assistant_result.id}")
        created_entities["assistant_id"] = create_assistant_result.id
    else:
        print("Error: Assistant creation failed:", create_assistant_result)
        rollback()
        exit()

    # Chat Completion Flow
    print("\n=== Chat Completion Flow ===")
    print(f"Asking assistant about the uploaded file '{file_name}'...")
    messages = ChatMessageList(messages=[
        ChatMessage(
            role="user",
            content=f"Please summarize the content of the {{file:{file_name}}} ({file_name}) in 50 words or less."
        )
    ])
    chat_response = chat_manager.chat_completion(
        model=f"saia:assistant:{assistant.name}",
        messages=messages,
        llm_settings=llm_settings
    )
    if hasattr(chat_response, "choices") and chat_response.choices:
        print(f"Success: Chat Response: {chat_response.choices[0].message.content}")
    else:
        print("Error: Chat completion failed:", chat_response)
        rollback()
        exit()

    print("\n=== Process Completed Successfully ===")


if __name__ == "__main__":
    try:
        main()
        rollback()
    except Exception as e:
        rollback()
        print(f"\n# Critical error: {e}")


from pygeai import logger
from pygeai.core.base.mappers import ErrorMapper, ResponseMapper
from pygeai.core.handlers import ErrorHandler
from pygeai.core.models import Project
from pygeai.core.base.responses import EmptyResponse
from pygeai.organization.clients import OrganizationClient
from pygeai.organization.mappers import OrganizationResponseMapper
from pygeai.organization.responses import AssistantListResponse, ProjectListResponse, ProjectDataResponse, \
    ProjectTokensResponse, ProjectItemListResponse
from pygeai.core.common.exceptions import APIError


class OrganizationManager:
    """
    Manager that operates as an abstraction level over the clients, designed to handle calls receiving and
    returning objects when appropriate.
    If errors are found in the response, they are processed to raise an APIError.
    """

    def __init__(self, api_key: str = None, base_url: str = None, alias: str = "default"):
        self.__organization_client = OrganizationClient(api_key=api_key, base_url=base_url, alias=alias)

    def get_assistant_list(
            self,
            detail: str = "summary"
    ) -> AssistantListResponse:
        """
        Retrieves a list of assistants with the specified level of detail.

        This method calls `OrganizationClient.get_assistant_list` to fetch assistant data
        and maps the response using `OrganizationResponseMapper` into an `AssistantListResponse` object.

        :param detail: str - The level of detail to include in the response. Possible values:
            - "summary": Provides a summarized list of assistants. (Default)
            - "full": Provides a detailed list of assistants. (Optional)
        :return: AssistantListResponse - The mapped response containing the list of assistants.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.get_assistant_list(detail=detail)
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while retrieving assistant list: {error}")
            raise APIError(f"Error received while retrieving assistant list: {error}")

        result = OrganizationResponseMapper.map_to_assistant_list_response(response_data)
        # TODO -> Add assistant list from plugins API
        return result

    def get_project_list(
            self,
            detail: str = "summary",
            name: str = None
    ) -> ProjectListResponse:
        """
        Retrieves a list of projects with the specified level of detail and optional filtering by name.

        This method calls `OrganizationClient.get_project_list` to fetch project data
        and maps the response using `OrganizationResponseMapper` into a `ProjectListResponse` object.

        :param detail: str - The level of detail to include in the response. Possible values:
            - "summary": Provides a summarized list of projects. (Default)
            - "full": Provides a detailed list of projects. (Optional)
        :param name: str, optional - Filters projects by name. If not provided, all projects are returned.
        :return: ProjectListResponse - The mapped response containing the list of projects.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.get_project_list(
            detail=detail,
            name=name
            )
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while retrieving project list: {error}")
            raise APIError(f"Error received while retrieving project list: {error}")

        result = OrganizationResponseMapper.map_to_project_list_response(response_data)
        return result

    def get_project_data(
            self,
            project_id: str
    ) -> ProjectDataResponse:
        """
        Retrieves detailed data for a specific project.

        This method calls `OrganizationClient.get_project_data` to fetch project details
        and maps the response using `OrganizationResponseMapper` into a `ProjectDataResponse` object.

        :param project_id: str - The unique identifier of the project to retrieve.
        :return: ProjectDataResponse - The mapped response containing project details.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.get_project_data(
            project_id=project_id
        )
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while retrieving project data: {error}")
            raise APIError(f"Error received while retrieving project data: {error}")

        result = OrganizationResponseMapper.map_to_project_data(response_data)
        return result

    def create_project(
            self,
            project: Project
    ) -> ProjectDataResponse:
        """
        Creates a new project with the given details and optional usage limit settings.

        This method calls `OrganizationClient.create_project` to create a new project and maps the response
        using `OrganizationResponseMapper` into a `ProjectDataResponse` object.

        :param project: Project - The project object containing details such as name, email, and description.
        :return: ProjectDataResponse - The mapped response containing the created project details.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.create_project(
            name=project.name,
            email=project.email,
            description=project.description,
            usage_limit={
                "subscriptionType": project.usage_limit.subscription_type,
                "usageUnit": project.usage_limit.usage_unit,
                "softLimit": project.usage_limit.soft_limit,
                "hardLimit": project.usage_limit.hard_limit,
                "renewalStatus": project.usage_limit.renewal_status,
            } if project.usage_limit is not None else None,
        )

        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while creating project: {error}")
            raise APIError(f"Error received while creating project: {error}")

        result = OrganizationResponseMapper.map_to_project_data(response_data)
        return result

    def update_project(
            self,
            project: Project
    ) -> ProjectDataResponse:
        """
        Updates an existing project with the provided details.

        This method calls `OrganizationClient.update_project` to update project information and maps the response
        using `OrganizationResponseMapper` into a `ProjectDataResponse` object.

        :param project: Project - The project object containing updated details such as project ID, name, and description.
        :return: ProjectDataResponse - The mapped response containing the updated project details.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.update_project(
            project_id=project.id,
            name=project.name,
            description=project.description
        )
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while updating project: {error}")
            raise APIError(f"Error received while updating project: {error}")

        result = OrganizationResponseMapper.map_to_project_data(response_data)
        return result

    def delete_project(
            self,
            project_id: str
    ) -> EmptyResponse:
        """
        Deletes a project by its unique identifier.

        This method calls `OrganizationClient.delete_project` to remove a project and maps the response
        using `ResponseMapper.map_to_empty_response`.

        :param project_id: str - The unique identifier of the project to be deleted.
        :return: EmptyResponse - An empty response indicating successful deletion.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.delete_project(
            project_id=project_id
        )
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while deleting project: {error}")
            raise APIError(f"Error received while deleting project: {error}")

        result = ResponseMapper.map_to_empty_response(response_data or "Project deleted successfully")
        return result

    def get_project_tokens(
            self,
            project_id: str
    ) -> ProjectTokensResponse:
        """
        Retrieves a list of tokens associated with a specific project.

        This method calls `OrganizationClient.get_project_tokens` to fetch token data and maps the response
        using `OrganizationResponseMapper.map_to_token_list_response`.

        :param project_id: str - The unique identifier of the project whose tokens are to be retrieved.
        :return: ProjectTokensResponse - The mapped response containing the list of project tokens.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.get_project_tokens(
            project_id=project_id
        )
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while retrieving project tokens: {error}")
            raise APIError(f"Error received while retrieving project tokens: {error}")

        result = OrganizationResponseMapper.map_to_token_list_response(response_data)
        return result

    def export_request_data(
            self,
            assistant_name: str = None,
            status: str = None,
            skip: int = 0,
            count: int = 0
    ) -> ProjectItemListResponse:
        """
        Exports request data based on specified filters.

        This method calls `OrganizationClient.export_request_data` to retrieve request data
        filtered by assistant name, status, and pagination parameters. The response is mapped
        using `OrganizationResponseMapper.map_to_item_list_response`.

        :param assistant_name: str, optional - Filters requests by assistant name. If not provided, all assistants are included.
        :param status: str, optional - Filters requests by status. If not provided, all statuses are included.
        :param skip: int, optional - The number of records to skip for pagination. Default is 0.
        :param count: int, optional - The number of records to retrieve. Default is 0 (no limit).
        :return: ProjectItemListResponse - The mapped response containing the exported request data.
        :raises APIError: If the API returns errors.
        """
        response_data = self.__organization_client.export_request_data(
            assistant_name=assistant_name,
            status=status,
            skip=skip,
            count=count
        )
        if ErrorHandler.has_errors(response_data):
            error = ErrorHandler.extract_error(response_data)
            logger.error(f"Error received while exporting request data: {error}")
            raise APIError(f"Error received while exporting request data: {error}")

        result = OrganizationResponseMapper.map_to_item_list_response(response_data)
        return result



from unittest import TestCase
import unittest
from pygeai.lab.managers import AILabManager
from pygeai.lab.models import SharingLink
from pygeai.core.common.exceptions import MissingRequirementException

ai_lab_manager: AILabManager

class TestAILabCreateSharingLinkIntegration(TestCase):  

    def setUp(self):
        self.ai_lab_manager = AILabManager(alias="beta")
        self.project_id = "be4889df-cacc-4e6f-b3bb-153c4ac0d168"
        self.agent_id = "0026e53d-ea78-4cac-af9f-12650e5bb6d9" 

    def __create_sharing_link(self, agent_id=None, project_id=None):
        return self.ai_lab_manager.create_sharing_link(
            project_id=self.project_id if project_id is None else project_id,
            agent_id=self.agent_id if agent_id is None else agent_id
        )
    

    def test_create_sharing_link(self):    
        shared_link = self.__create_sharing_link()        
        self.assertIsInstance(shared_link, SharingLink, "Expected response to be an instance of SharingLink")
       
        self.assertEqual(
            shared_link.agent_id,
            self.agent_id,
            "Returned agentId should match the requested agent_id"
        )
        self.assertTrue(
            shared_link.api_token.startswith("shared-"),
            "apiToken should start with 'shared-'"
        )
        self.assertTrue(
            shared_link.shared_link.startswith("https://"),
            "sharedLink should be a valid URL"
        )
        self.assertIn(
            f"agentId={self.agent_id}",
            shared_link.shared_link,
            "sharedLink should contain the agentId as a query parameter"
        )
        self.assertIn(
            f"sharedToken={shared_link.api_token}",
            shared_link.shared_link,
            "sharedLink should contain the apiToken as sharedToken"
        )


    def test_create_sharing_link_no_project(self):
        with self.assertRaises(MissingRequirementException) as context:
            self.__create_sharing_link(project_id="")
        self.assertEqual(
            str(context.exception),
            "Cannot create sharing link without specifying project_id",
            "Expected exception for missing project_id"
        )


    def test_create_sharing_link_no_agent_id(self):
        with self.assertRaises(MissingRequirementException) as context:
            self.__create_sharing_link(agent_id="")
        self.assertEqual(
            str(context.exception),
            "agent_id must be specified in order to create sharing link",
            "Expected exception for missing agent_id"
        )

    @unittest.skip("A descriptive error exception is expected")
    def test_create_sharing_link_invalid_agent_id(self):
        invalid_id = "0026e53d-ea78-4cac-af9f-12650invalid"
        shared_link = self.__create_sharing_link(agent_id=invalid_id)

        """ self.assertEqual(
            deleted_agent.content["messages"][0]["description"],
            f"Agent not found [IdOrName= {invalid_id}].",
            "Expected error message for invalid agent id"
        ) """


    def test_create_sharing_link_invalid_project_id(self):
        invalid_id = "0026e53d-ea78-4cac-af9f-12650invalid"
        shared_link = self.__create_sharing_link(project_id=invalid_id)

        self.assertIsInstance(shared_link, SharingLink, "Expected response to be an instance of SharingLink")
       
        self.assertEqual(
            shared_link.agent_id,
            self.agent_id,
            "Returned agentId should match the requested agent_id"
        )
        self.assertTrue(
            shared_link.api_token.startswith("shared-"),
            "apiToken should start with 'shared-'"
        )
        self.assertTrue(
            shared_link.shared_link.startswith("https://"),
            "sharedLink should be a valid URL"
        )


from pygeai.lab.managers import AILabManager
from pygeai.lab.models import Task

manager = AILabManager()
task = Task(name="basic-task-4", description="Basic task for process", title_template="Basic Task")
result = manager.create_task(task=task, automatic_publish=True)

print(f"Created task: {result.name}, ID: {result.id}")
